<?php
defined( 'ABSPATH' ) or exit;   // show a blank page if try to access this file directly

require_once( EWZ_PLUGIN_DIR . 'classes/ewz-base.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-exception.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-field.php' );
require_once( EWZ_PLUGIN_DIR . 'classes/ewz-layout.php' );
require_once( EWZ_RATING_DIR . 'classes/ewz-rating-form.php' );
require_once( EWZ_RATING_DIR . 'classes/validation/ewz-rating-form-input.php' );
require_once( EWZ_PLUGIN_DIR . 'includes/ewz-common.php' );
require_once( EWZ_CUSTOM_DIR . 'ewz-custom-data.php' );


/**
 * Process input data 
 **/
function ewz_process_rating_form_input()
{
    switch( $_POST['ewzmode'] ){
        // 'spread' is caught by the ewz_
        // function in ewz_admin-rating.php, which is hooked to plugins_loaded

    case  'ratingform':

        // validate all input data ( except uploaded files )
        $input = new Ewz_Rating_Form_Input($_POST);
        $ratingform = new Ewz_Rating_Form( $input->get_input_data() );
                              // text fields are sanitized before saving
        $ratingform->save();
        break;

    default:
        throw new EWZ_Exception( 'Invalid Input ', 'mode=' . preg_replace( '/[^a-z0-9 _-]/i', '_', $_POST['ewzmode'] ) );
    }
}


/**
 * Callback for add_submenu_page
 * 
 * Actually outputs the html for the ratings admin page
 *
 **/
function ewz_rating_form_menu(){
    $message = '';
    if ( !Ewz_Rating_Permission::can_see_rating_form_page() ) {
        wp_die( "Sorry, you do not have permission to view this page" );
    }

    if( isset( $_POST['ewzmode'] ) ){
      try {
          ewz_process_rating_form_input();
      } catch( Exception $e ) {
            $message .= $e->getMessage();
            $message .= "\n";
      }
    }
    try {

        $s_options = Ewz_Rating_Scheme::get_rating_scheme_opt_array();
        array_unshift( $s_options, array('value'=>'', 'display'=>'-- Select Rating Scheme --' ));
        $schemes_list = ewz_option_list( $s_options );
        $rating_forms = Ewz_Rating_Form::get_all_rating_forms( 'can_edit_rating_form_obj' );

        $judge_list = Ewz_Rating_Form::get_judge_opt_array();
        foreach(  $rating_forms as $rform ){
            $rform->add_display_info( $judge_list );
        }

        /*******************************/
        /* Pass the data to Javascript */
        /*******************************/
        $ewzG = array( 'rating_forms' => $rating_forms );
        $ewzG['ajaxurl'] = admin_url('admin-ajax.php', (is_ssl() ? 'https' : 'http'));
        $ewzG['load_gif'] = plugins_url( 'images/loading.gif', dirname(__FILE__) ) ;
        $ewzG['userlist'] = ewz_option_list(  $judge_list);   // judge list for new rating forms
        $ewzG['message'] = wp_kses( $message, array( 'br' => array(), 'b' => array() ) );
        $ewzG['helpIcon'] = plugins_url( 'images/help.png', dirname( __FILE__ ) );
        $ewzG['webforms'] = ewz_html_esc( array_values( Ewz_Webform::get_webform_titles('can_view_webform' )) );
        $ewzG['schemes'] = ewz_html_esc( Ewz_Rating_Scheme::get_all_rating_schemes() );
        $ewzG['schemes_list'] =  $schemes_list;
        $ewzG['nonce_string'] = wp_nonce_field( 'ewzadmin', 'ewznonce', true, false );
        $ewzG['jsvalid'] = Ewz_Base::validate_using_javascript();
        $ewzG['errmsg'] = array(
                'warn' => '*** WARNING ***' ,
                'reallydelete' => 'Really delete the entire rating form?',
                'noundo' => 'This action cannot be undone',
                'hasitems' => 'Deleting this rating form will also delete ALL its associated ratings.',
                'webform' => 'At least one webform must be selected. You may narrow the selection further once that is saved.',
                'judge' => 'At least one judge must be selected in order to open the rating form.',
                'formTitle' => 'Please enter a title for the rating form.',
                'formIdent' => 'Each rating form must have an identifier that starts with a lower case letter
                    and consists only of lower case letters, digits, dashes and underscores.'
            );


        // use of $ewzG1.var = ewzG  is a hack to get around the fact that wp_localize_script
        // runs html_entity_decode on scalar values.  Our data is already processed where it needs to be,
        // and some of it contains html entities which should not be decoded.
        wp_localize_script( 'ewz-admin-rating-forms', 'ewzG1',  array( 'gvar'=>$ewzG ) );

    ?>
 <div class="wrap">
    <h2>EntryWizard Rating Form Management</h2>
    <p><img alt="Help icon" class="ewz_ihelp" src="<?php print  $ewzG['helpIcon']; ?>" onClick="ewz_help('rforms')"> &nbsp;
        A rating form may be inserted into any page using the shortcode &nbsp;
         <b>&#91;ewz_show_rating&nbsp;&nbsp;identifier="xxx"&#93;</b>
         &nbsp; where xxx is the identifier you created for the form
         <br>( NOTE: ewz_show_upload and ewz_show_rating shortcodes may <b>not</b> be mixed on the same page ) </p>

    <div id='rorder'>Rating forms may be dragged to rearrange.   &nbsp; &nbsp; &nbsp; 
         <button type="button" id="rating_forms_save1_" onClick="save_ratingform_order()"> Save Order of Rating Forms </button> 
         &nbsp; &nbsp; &nbsp; &nbsp; 
         <button  type="button" class="button-secondary" id="rforms_sort" onClick="sort_alpha()">Sort Rating Forms Alphabetically</button>
         <div class="ewz_numc"><?php print $ewzG['nonce_string']; ?></div>
    </div>
    

    <div id="ewz_actions">
       <div id="ewz_all_check"><input type="checkbox" id="ewz_all_forms" onClick="toggle_select_all_forms()"> Select All</div>
       <div id="ewz_bulk">
           <select name="action" id="bulk-action-selector">
              <option value="-1">Bulk Actions</option>
              <option value="ewz_batch_delete_rforms">Delete All Selected Rating Forms</option>
              <option value="ewz_batch_delete_ratings">Delete Ratings From Selected Rating Forms</option>
           </select> 
           <button  type="button" id="dobatch" onclick="do_batch()">Apply</button>
       </div>
   </div> 
         

    <div id="ewz_management">
         <br>
    </div>

    <!-- HELP SECTION Rating Forms ~ Rating Form Management Page  -->
    <div id="help-text" style="display:none">
       <!-- HELP POPUP Rating Forms -->
       <div id="rforms_help" class="wp-dialog ewz-help" >
          <p>The EntryWizard "Rating" system is designed for viewing and/or judging the uploaded images. </p>
          <p>The Rating Form is generated by the 'ewz_rating' shortcode:  <b>&#91;ewz_show_rating&nbsp;&nbsp;identifier="xxx"&#93;</b>
         &nbsp; where xxx is the identifier you created for the rating form  
         ( note that there must be <u>no space beween the opening square bracket and the shortcode name</u> )</p>
         <p>NOTE 1: ewz_show_upload and ewz_show_rating shortcodes may <b>not</b> be mixed on the same page </p>
         <p>NOTE 2: Depending on your web hosting setup, there may be limitations on how much data a judge can save at one time. 
            Judges should be reminded to save frequently.  A Settings option allows for a popup reminder to do this. </p>
            <p>NOTE 3: Judges (and administrators) should always refresh the page after a period of inactivity, before doing any more work.</p>
             <p>-----------------</p>
          <p>A Rating Scheme describes the format of the rating form the way a layout describes the format of a WebForm, and each Rating Form must be assigned a previously-created Rating Scheme.
             Because it deals with the fields filled out when the image was uploaded, a rating scheme must be associated with a specific layout, and may only be used for items uploaded using that layout.</p>
          <p>Rating forms may be dragged to rearrange.  Click "Save Order of Rating Forms" to preserve the order for your next visit.</p>
             <p>The Rating Form is visible to, and editable by, anyone who has EntryWizard permissions to assign the layout associated with its Rating Scheme.  Downloading the spreadsheets requires permission to download from any webform with the associated layout, or permission to edit the rating form itself.</p>
             <p>In addition, as of version 1.1.4, permission to edit and download from any individual rating form may be set on the Permissions page.</p>
             <p>-----------------</p>
             <p>There are also some more parameters that may be added to the shortcode if you wish.  Here is the full list of parameters:
             <ul class="ewz_lpad"><li><b>identifier</b> e.g. identifier="competition2"  -- the identifier you created for the rating form.  
                     This parameter is required in all cases.</li>
             <li><b>item_ids</b> e.g. item_ids="256,278,300"  -- a comma-separated list of wordpress item_ids. 
                     If this parameter is present, only the listed items will be displayed. 
                     The easiest way to obtain the item_ids is to include "WP Item ID" in your spreadsheet, using the 
                     "Extra Data For Display in Spreadsheet" section of the Rating Scheme page.</li>
             <li><b>judge_ids</b> e.g. judge_ids="256,278,300" 
                  -- a comma-separated list of user_ids, which must be the Wordpress ID's of judges with access to the rating form.
                 If this parameter is present, judges <u>not</u> in the list will see nothing at all ( not even a "you do not have permission" message).
                 If it is not present, all judges specified in the rating form will be allowed to see the form. 
                 <br>Judge user_id's are shown in the judge selection box. </li>
             <li><b>rf_num</b>  e.g. rf_num="2" -- this parameter is <u>required if more than one ewz_show_rating
                  shortcode is present</u> on the page.<br>
                 Each shortcode must have a different rf_num parameter.</li>
             <li><b>view</b>   view="read", view="secondary" or view="rate"  -- specifies if the view is to allow input or just be "read-only".
                 "read" creates a totally read-only view. 
                  "secondary" creates a view where only fields with the "secondary" box checked may be changed, the rest are read-only. 
                  The default is "rate", which is the normal view showing all fields except those with the  "secondary" box checked. If there is no
                  "view" parameter, "rate" is assumed.<br>
                 The "divide between judges" field is ignored for the "read" view, but honoured for "rate" and "secondary" views.</li>
             <li><b>others</b> (Starting with version 1.1.3) Has only one possible value:  others="true".  
                 If this is set, ratings for all other judges will also be displayed (in read-only form). <br>
                 With 5 or fewer judges, a line will be displayed for each judge.  If there are more than 5 judges, only non-blank ratings will show.
                 <br><b>NOTE:</b>As of version 1.1.3, this parameter is required as well as 'view="read"' to obtain the view previously shown by 'view="read"'.</li>
             <li><b>finish</b> (Starting with version 1.1.9) finish="show" or finish="hide" controls whether the "finish" button is displayed to the judge.
                 If this parameter is not set, the default value from the rating scheme is used. </li>
            </ul>
              </p>
                 <p>When you edit a page (Visual tab), you should see a drop-down list of "EWZR" rating forms. 
                 Choose the "simple" option if you are just creating a single shortcode on  the page, with the normal "rate" view  and no 
                 limitations of items or judges beyond those set on the rating_form page.<br>  
                 For anything else, choose the "general" option and it will ask you for all the required inputs.
              </p>  
              <p><u>Always check the final page view</u>, just in case of errors in the shortcode, item selection, judge assignment, etc. 
                  
                 Since the person creating the rating form would not normally be a judge, it would be very useful to have a plugin installed
                 that allows an administrator to see the site as another user ( e.g. User Switching ). 
             </p>
             <p> &nbsp;
             </p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Title -->
       <div id="title_help" class="wp-dialog ewz-help" >
            <p>Each rating form needs a title which will be displayed at the top of the form.</p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Identifier -->
       <div id="ident_help" class="wp-dialog ewz-help" >
                 <p>Each rating form also needs a short (no more than 15 characters) but unique identifier which is used in the shortcode. </p>
            <p>It must consist of letters, digits or underscores only ( no spaces ).</p> 
            <p>No two rating forms may have the same identifier.</p>
            
       </div>
       <!-- HELP POPUP Rating Forms ~ Item Selection -->
       <div id="iselect_help" class="wp-dialog ewz-help" >
            <p>In this section, you select the items that  will be displayed in the rating form.  </p>
            <p>The items <b>must</b> have been uploaded via a webform using the <b>same layout</b> that was used for this rating form's scheme.</p>
            <p>All webforms using that scheme are listed here, and you are required to select one or more of them.
               You may optionally check "User's own images only". 
            </p>
            <p>Once the rating form has been saved, you will also be able to restrict the images to just items that 
               match any drop-down options associated with the <u>layout used for the image upload</u>.  
               ( You may find that using these options takes 
               more resources on the server. ) Currently only fields that are drop-down option lists are available here.
            </p>
            <p>Items will normally be displayed ordered by their wordpress item_id ( usually the order in which they were uploaded ).<br>
               This will often result in all or most of one person's images being shown consecutively.<br>
               If you check "Shuffle Item Order", a new number is generated from the item_id by "moving" the last digit to the start,
               and the items are sorted by this number. <br>This is not a random shuffling, but it does create a predictable order 
               that can be repeated if a judge refreshes a page.
            </p> 
            <p>It is inadvisable to change any of these settings once any judges have created ratings.</p>               
       </div>
       <!-- HELP POPUP Rating Forms ~ dq -->
       <div id="dq_help" class="wp-dialog ewz-help" >
           <p>Not everyone will need this. It allows you to flag some items as "disqualified", so that they remain in the entrywizard database, 
              but are <u>not shown to the judges</u>.  Potentially useful if you need to keep a record of such items.      
           </p>
           <p>Enter here the Item_ids of the disqualified items, separated by commas, like this: 1234,950,1235,1106</p>
           <p>The listed items will not be displayed to judges, but one line for each <u>will</u> appear in the downloaded spreadsheet.
              If you select to display the item_id's in the spreadsheet, those for disqualified items will be preceeded by 'DQ '.
           </p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Access -->
       <div id="access_help" class="wp-dialog ewz-help" >
            <p>Here you select who may see the rating form.  "All Logged-In Users" would not normally be 
               checked for a real "judging".  
               You would normally use it in combination with "Users Own Images" above, to show users a list of items they have uploaded. 
            </p>
            <p>Installing EntryWizard automatically creates a new Wordpress role 'EntryWizard Judge'. You may 
               select one or more users who have been assigned that role, and they will see all items matching the item selection above.
            </p>
            <p>Since the person creating the rating form would not normally be a judge, it would be very useful to have a plugin installed
               that allows an administrator to see the site as another user ( e.g. User Switching ). 
            </p>
            <p>If you wish to use already-registered users as judges, you may also find it helpful to install a plugin that lets you 
               give multiple roles to users. But if you do, be careful not to take away the 'EntryWizard Judge' role from an active judge.
            </p>
            <p>The "ID" values displayed in the judge selection box are the values assigned by Wordpress.  You may need them if you
                wish to create any "tie breaker" or other such forms limited to particular judges ( see the general Rating Form help 
               item at the top of this page ).
            </p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Open -->
       <div id="open_help" class="wp-dialog ewz-help" >
           <p>When this is checked, users with the right access will be able to see the rating form.  Otherwise 
              they will see just a message saying the form is not currently open.
           </p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Data Management -->
       <div id="data_help" class="wp-dialog ewz-help" >
           <p>When judges with "EntryWizard Judge" role have been selected, this area shows a summary of how many items 
              they have rated, and whether they have finished.</p>
           <p>When "All Logged-in Users" has been selected under Access Control, the area simply shows the total number of ratings saved by all users.
              This will be 0 if the form is for display only, and contains no input fields</p>
           <p>Because updating this puts a load on the server, what you see may not always be up-to-the-minute.  
              When you require exact information, click "Recalculate".
           </p>
           <p>You may, if necessary, delete all of a judge's ratings for this rating-form.</p>
           <p>You may also, if necessary, temporarily or permanently close the rating form for this judge only, by clicking "Force Finish". (This will cause the status to display as "Judge Finished", even if all items have not been judged.)</p>
           <p>If a judge has indicated they are finished, but you wish them to make further changes, click the "reopen" button 
                  to allow them access again.</p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Overwrite -->
       <div id="overwrite_help" class="wp-dialog ewz-help" >
          <p>"Overwrite" options display only for fields that have been set to "secondary" in the rating scheme.</p>
          <p>They allow you to download a spreadsheet with values in, for instance, a "score" column overwritten by those in a secondary 
             "re-score" column if they exist for the same judge.</p>.
       </div>

       <!-- HELP POPUP Rating Forms ~ Download -->
       <div id="download_help" class="wp-dialog ewz-help" >
           <p>When there is more than one judge, there are two possible formats for the downloaded spreadsheet.  
                  <ol><li><b>1 row per rating: </b>If there are 3 judges, each uploaded item will get 3 rows in the spreadsheet, one for each judge's inputs.</li>
                      <li><b>1 row per item: </b>There is only one row for each uploaded item, and extra columns are added for the judges' inputs.
                  To use this format, your rating scheme <u>must allow a blank column for each additional judge</u> immediately after each judge input column.<br>
                  For instance, if you have 3 judges, with each being asked for a score and a comment, the two columns right after your score column must be blank, 
                  and the two columns right after your comment column must be blank
                     </li>
                  </ol>
            </p>
            <p>With only one judge, the two formats should look the same. </p>
            <p>If the "divide" flag has been set for a field in the rating scheme, the id of the judge to whom it was displayed is shown in parentheses beside 
                  the field, or, if the judge has their own column, a star is shown beside the field. </p>
            <p>If "All Logged-in Users" has been selected under Access Control, only the "1 row per rating" format is available. </p>
       </div>

       <!-- HELP POPUP Rating Forms ~ Add / Arrange Rating Forms -->
       <div id="rfsort_help" class="wp-dialog ewz-help" >
                  <p>Add a New Rating Form creates a new Rating Form for the Rating Scheme you select 
                     ( provided there is at least one Webform associated with it ). You then need to:
              <ul class="ewz_lpad"><li>Give it a name and identifier</li>
                  <li>Select the WebForm(s) containing the items to be rated.</li>
                  <li>Select who is to be allowed access.</li>
                  <li>Check the "Open Rating" box when you are ready to allow rating.</li>
                  <li>Save your changes.</li>
                  <li>After saving, you may be able to refine your item selection, depending on the layout.</li>
              </ul>
           </p>
           <p> Rating Forms may be dragged up or down to rearrange them. Clicking "Save Order of Rating Forms" will save the order, 
               and the new order will subsequently be used for this page.
           </p>    
       </div>
    </div><!-- help -->
</div><!-- wrap -->

     <?php

       } catch( Exception $e ){
            wp_die( $e->getMessage() );
    }
} // end function ewz_rating_form_menu


