<?php
defined( 'ABSPATH' ) or exit;   // show a blank page if try to access this file directly
/****************************/
/* Generate the rating form */
/****************************/
require_once( EWZ_PLUGIN_DIR . 'includes/ewz-common.php' );

/**
 * Return a string used to display a single field, either input or read-only
 *
 * @param  $args   array of named args
 *   user_id       int                 ID of current user ( judge )
 *   rating_form   Ewz_Rating_Form     The rating_form in which the field is to be displayed. 
 *   rating_field  Ewz_Rating_Field    The rating_field to be displayed
 *   item_id       integer             The item to be rated
 *   ratings       array of Ewz_Item_Ratings   The item ratings to be displayed
 *   rownum        integer             The row number of the item
 *   append_next   boolean             True if the next field is to be appended to this one
 *   atts          array               attributes for the shortcode
 *
 * @return string used to display the value or input
 **/
function ewz_display_rated_item_field( $args ){   //
    // assert( is_array($args) );
    $user_id = $args['user_id']; 
    $rating_form = $args['rating_form'];
    $rating_field = $args['rating_field'];
    $item_id = $args['item_id'];
    $ratings = $args['ratings'];
    $rownum = $args['rownum'] ;
    $append_next = $args['append_next'];
    $thumb_w = $args['thumb_w']; 
    $judge_index = $args['judge_index'];
    $atts = $args['atts'];

    // assert( Ewz_Base::is_nn_int( $user_id ) );
    // assert( Ewz_Base::is_nn_int( $item_id ) );
    // assert( is_a( $rating_form, 'Ewz_Rating_Form' ) );
    // assert( is_a( $rating_field, 'Ewz_Rating_Field' ) );
    // assert( is_array( $ratings ) );
    // assert( is_int( $rownum ) );
    // assert( is_bool( $append_next ) );
    // assert( is_array( $atts ) );
    // assert( in_array( 0, $rating_form->judges ) || isset( $rating_form->show_judge[$item_id] ));
    // assert( !( in_array( 0, $rating_form->judges ) && $rating_form->divide_a_field ) );

    // dont show a secondary field at all unless view is secondary
    if( $rating_field->is_second && $atts['view'] != 'secondary' ){
        return '';
    }

    $_rf_num = esc_attr( $atts['rf_num'] );
    $item = new Ewz_Item($item_id);
    //NB: a field may be divided, but that is ignored if divide_a_field is false ( ie the judge_list contains 0)
    $field_hidden = ( $rating_form->divide_a_field &&
                      $rating_field->divide && 
                      isset($rating_form->show_judge[$item_id]) &&
                     ( $rating_form->show_judge[$item_id] != $user_id ) &&
                     !$atts['others'] );
    $fixed_field = in_array( $rating_field->field_type, array( 'fix', 'xtr', 'lab' ) );
   
    $_value = '';
    $prefix = '';
    if( !$field_hidden ) {
        $same_for_all = $fixed_field && !$rating_field->divide;
        foreach( $ratings as $n => $rating ){
        // only show one if all judges see the same
            if( $same_for_all && $n > 0 ){
                continue;
            }
            // is it an actual input field for this judge in this view?
            $is_input = !$fixed_field 
                        &&
                       ( $atts['view'] == 'rate' || ( ( $atts['view'] == 'secondary' ) && $rating_field->is_second ) )
                        &&
                       $n == $judge_index;

            $show_prefix = $atts['others'] && !$same_for_all && !$is_input;

            if( $show_prefix ){ 
                $prefix =  '<span class="ewzj">' . (  $n == $judge_index  ?  'Self' : $rating->judge_id ) . ':&nbsp;</span>';
            }

            // newline between judges in multi-judge view
            if( $_value && substr($_value,-4) != '<br>' ){
                $_value .= '<br>';
            }
            if( $is_input ){
                // an actual input for this judge
                $field_display = Ewz_Item_Rating::rating_field_display( $rating_form, $rating_field, $item, $rating );
                                                                             // array( stored_value, display_value )
                $cleanval = esc_attr( wp_strip_all_tags( $field_display[0] ) ); 
                $_value .= ewz_input_display( $rating_field, $atts, $rating,  $cleanval, false );

            } else {
                // a read-only value.
                $field_hidden_from_judge = $rating_field->divide && ( $rating_form->show_judge[ $item_id ] != $rating->judge_id );
                
                $display_value = Ewz_Item_Rating::rating_field_display( $rating_form, $rating_field, $item, $rating )[1];

                // ignore ratings where the field is hidden from the judge
                if( !$field_hidden_from_judge ){
                    $_value .=  $prefix . ewz_safe_tags( $display_value );
                }   
                // for image inputs, modify this to display the thumbnail
                if( preg_match( '/^http/', $_value ) ){ 
                    $filename = $item->item_files[$rating_field->fdata['field_id']]['fname'];
                    $_value = ewz_add_images( $_value, $filename, $_rf_num, $rownum, $rating_field, $thumb_w );
                }
            }
        }
    } 
    // assert( is_string( $_value ) || ( null == $_value  )|| ( '' == $_value  ) );

    // add an identifier so we can pick out appended items
    $tdstring = '';

    if( !$rating_field->append ){
        $tdstring = "<td id='cell{$_rf_num}_{$item_id}_". esc_attr( $rating_field->rating_field_id ) . "'  ts_custom=''>";
    }

    $m =  strpos($_value, 'ewzsorton') == false ? " ewzsorton='txt' " : '';
    $tdstring .= "<span $m class='ewzval_" . esc_attr( $rating_field->rating_field_id ) . "'>" . $_value . "</span>";

    if( $append_next ){
        if( substr($tdstring,-4) != '<br>' ){
            $tdstring .= "<br>";
        }
        // possible multiple values for appended fields, show an hr to separate them
        if( $atts['others'] && (!$fixed_field || $rating_field->divide ) ){
             $tdstring .= '<hr class="ewz_divider">';
        }
    } else {
        $tdstring .= '</td>';
    } 
    return $tdstring;
}        // end display_rated_item_field


function  ewz_add_images( $in_value, $filename, $rf_num, $rownum, $rating_field, $thumb_w  )
{
    if ( is_file( $filename)){
        $imgmod_t = filemtime( esc_attr($filename) );
        $new_value = '<img id="img' . esc_attr( "{$rf_num}_{$rownum}_{$rating_field->rating_field_id}" ) . '" class="ewz_thumb" src="' . "${in_value}?$imgmod_t" . '"';
        $new_value .= ' onClick="ewz_image_window( ' . $rf_num . ', ' . esc_attr( $rownum ) . ', ' . $rating_field->rating_field_id .' )">';  
    } else {
        // image to use if required image not found
        $noImg = plugins_url( 'images/notFound.jpg', dirname(__FILE__) );   // thumbnail
        $new_value = '<img id="img' . esc_attr( "{$rf_num}_{$rownum}_{$rating_field->rating_field_id}" ) . '" width="' . $thumb_w .'" class="ewz_thumb" src="' . $noImg  . '"';
        $new_value .= ' onClick="ewz_image_window( ' . $rf_num . ', ' . esc_attr( $rownum ) . ', ' . $rating_field->rating_field_id .' )">';  
        $imgmod_t = 999;
        error_log("EWZ: Missing image file $filename" );
    }

    return $new_value;
}

/**
 * Return the html for the individual rating *page* for a single item
 * 
 * @param  $rating_fields    array of Ewz_Rating_Fields 
 * @param  $item_rating      the Ewz_Item_Rating to be filled out or displayed
 * @param  $item_row         integer --  position of item on page
 * @param  $rowcount         integer --  total number of items 
 *
 * @return 
 **/
function ewz_item_rating_page( $rating_fields, $item, $item_row, $settings, $rowcount, $atts ){
    // assert( is_array( $rating_fields ) );
    // assert( is_array( $settings ) );
    // assert( is_a( $item, 'Ewz_Item' ) );
    // assert( is_int( $item_row ) );
    // assert( is_int( $rowcount ) );
    // assert( Ewz_Base::is_nn_int( $atts['rf_num'] ) );

    $_rf_num = esc_attr( $atts['rf_num'] );
    $_item_row = (int)$item_row;

    $output = '';
    // the content of this div gets shown in the popup image window when the thumbnail is clicked
    foreach ( $rating_fields as $n => $rating_field ) {
        if( $rating_field->field_type == 'fix' ){ 
            // a read-only data field
            $item_field_id = $rating_field->fdata['field_id'];
            $item_field = new Ewz_Field($item_field_id);
            if ( $item_field->field_type == 'img' ) {
                $output .= ewz_main_image_display( $item, $item_field_id, $rating_field->rating_field_id, $_rf_num, $_item_row, $settings );
            }
        }        
    }
    return $output;
}


function ewz_main_image_display( $item, $item_field_id, $rating_field_id, $rf_num, $item_row, $settings ){
    // assert( is_a( $item, 'Ewz_Item') );
    // assert( is_int( $item_field_id ) );
    // assert( is_int( $rating_field_id ) );
    // assert( is_string( $rf_num ) );
    // assert( is_int( $item_row ) );
    // assert( is_array( $settings ) );
    // assert(is_numeric($settings['max_shrink']) && $settings['max_shrink'] <= 100 && $settings['max_shrink'] >= 0 );
    $esc_ident = esc_attr( "{$rating_field_id}_$rf_num$item->item_id" );

    $imgW = 0;
    $imgH = 0;
    $minImgW = 0;
    $minImgH = 0;
    
    if( isset( $item->item_files ) && $item->item_files ){
        $imgW = $item->item_files[$item_field_id]['width'];
        $imgH = $item->item_files[$item_field_id]['height'];
        $minImgW = ceil( $settings['max_shrink']*$imgW/100 );
        $minImgH = ceil( $settings['max_shrink']*$imgH/100 );
    }
    $noImg = plugins_url( 'images/notFound800.jpg', dirname(__FILE__) );
    if( $settings['maxw'] > 2000 && $settings['maxh'] > 2000 ){
        $noImg = plugins_url( 'images/notFound2000.jpg', dirname(__FILE__) );
    } elseif( $settings['maxw'] > 1200 &&  $settings['maxh'] > 1200 ){
        $noImg = plugins_url( 'images/notFound1200.jpg', dirname(__FILE__) );
    }
    $fname = "";
    $filename = "";
    if( isset( $item->item_files[$item_field_id] ) ){
        $filename = $item->item_files[$item_field_id]['fname'];
    } 
    if( $filename ){
        if(is_file( $filename ) ){     
            $imgmod_t = filemtime( esc_attr($filename) );
            $fname = esc_attr( ewz_file_to_url( $item->item_files[$item_field_id]["fname"] ) ) . "?$imgmod_t";
        } else {
            error_log("EWZ: Missing image file $filename" ); 
            $fname = "{$noImg}?999";
        }
    }
        
    $img_style = " min-width: {$minImgW}px;  min-height:{$minImgH}px;";

    // for browsers/OS's that dont support the resizing css, fix the size
    //global $is_winIE;
    
    $output = "<div class='ewz_rating_page_$esc_ident' style='display: none;'>";
    if( $fname ){
        $output .=  "<div id='imgcontainer$esc_ident' class='ewz_imgcntr' >   ";
        $output .=  "  <img id='mainimg$esc_ident' class='mainsized' onContextMenu='return false;'  src='$fname' style='$img_style' >   ";
        $output .=  "</div>   ";
    } else {
        $esc_h = esc_attr($settings['maxh'])/2;
        $output .=  "<div style='width: 100%; font-size: 1.2em; text-align: center; margin-top: {$esc_h}px;'>No Image</div>";
    }
    $btn_style = 'background: ' . $settings['fcol'] . '; color: ' .  $settings['bcol'] .';';
    $output .=  "<div id='imgnav$esc_ident' >   ";
    $output .=  "   <div class='ewz_img_btns'>";
    if($fname ){
    $output .=  "      <div class='ewz_toggle'><button type='button' class='ewz_link_btns' style='$btn_style' id='fullsize$esc_ident' onClick='opener.ewz_toggle_full_size(\"$esc_ident\")'>Toggle Full Size With Scrollbars</button></div>";
    }
    $output .=  "      <button type='button' class='ewz_link_btns' style='$btn_style' id='prevbtn$esc_ident' onClick='opener.ewz_image_window($rf_num, " . ($item_row - 1) . ", $rating_field_id)'>&lt;-- Previous</button>";
    $output .=  "      <button type='button' class='ewz_link_btns' style='$btn_style' id='testbtn$esc_ident' onClick='opener.ewz_show_image2($rf_num, $item_row, \"\" , $rating_field_id )'>Display A Test Image</button>";
    $output .=  "      <button type='button' class='ewz_link_btns' style='$btn_style' id='nextbtn$esc_ident' onClick='opener.ewz_image_window($rf_num, " . ($item_row + 1) . ", $rating_field_id)'> &nbsp; &nbsp; Next --&gt; </button>";
    $output .= "       <div class='ewz_size'>  </div>"; 
    $output .=  "  </div>";
    $output .=  "</div>";
    $output .=  "</div>";
    
    return $output;
}



/**
 * Call the EntryWizard display_...._form_field functions, accorting to the rating _field type
 *
 * @param Ewz_Rating_Field   $rating_field    the Ewz_Rating_Field to display
 * @param array              $rating          the Ewz_Item_Rating to be displayed
 * @param string             $savedval        data currently stored for the field
 * @param boolean            $fixed           -- true if an option may not be changed ( because it is used in a restriction )
 **/
// a parameter webform_id is required for these functions ( in ewz_common.php )
// to allow for multiple webforms appearing on a single page.  It is only used to generate an id.     
// Using functions from the upload code. If img or radio items added, will need to fix the
// fact that those functions require $rating_field to have a "field_id"
 
function ewz_input_display( $rating_field, $atts, $rating,  $savedval, $fixed ){
    // assert( is_a( $rating_field,  'Ewz_Rating_Field' ) );
    // assert( is_array( $atts ) );
    // assert( is_a( $rating, 'Ewz_Item_Rating' ) );
    // assert( is_string( $savedval ) );
    // assert( is_bool( $fixed ) );
    // assert( in_array( $rating_field->field_type, array( 'str', 'opt', 'rad', 'chk' ) ) );
    $_rf_num = $atts['rf_num'];

    $_id    = esc_attr( "rating". $atts['rf_num'] . "_" . $rating->item_id . '_' . $rating_field->rating_field_id . "_" );
    $_name  = esc_attr( "rating[" . $rating_field->rating_field_id . "]" );
    $_savedval = ewz_safe_tags( "$savedval" );

    // inputs generated from the ewz functions do not have the 'ewz_ts_set' function triggers required for sorting, must add them in
    // This assumes there is not more than one ">" in the generated code.
    // Note the "limit" arg of 1 to preg_replace -- only replace the first occurrence
    switch ( $rating_field->field_type ) {
    case 'str': $_value = ewz_display_str_form_field( $_name, $_id, $_savedval, $rating_field );
        // nextafter never used for text field -- no way to tell when input finished
        $display =  preg_replace( '/>/', " ewzsorton='val' onInput='ewz_ts_set(" . $_rf_num . ", this)'>",$_value, 1);
        break;
    case 'opt': $_value = ewz_display_opt_form_field( $_name, $_id, $_savedval, $rating_field, (bool)$fixed );
        // Pass field-id to ewz_ts_set in case nextafter is set
        $display = preg_replace( '/>/', " ewzsorton='val' onChange='ewz_ts_set(" . $_rf_num . ", this,$rating_field->rating_field_id)'>", $_value, 1);
        break;

        // No 'rad' fields -- makes no sense for large numbers of items

    case 'chk': // savedval is '-' for display of "unchecked" 
        // Pass field-id to ewz_ts_set in case nextafter is set
        $_value = ewz_display_chk_form_field( $_name, $_id, str_replace( '-', '', $_savedval) );
        $display = preg_replace( '/>/', " ewzsorton='chk' onChange='ewz_ts_set(" . $_rf_num . ", this,$rating_field->rating_field_id)'>", $_value, 1);
        break;
    default:
        throw new EWZ_Exception( "Invalid field type " . $rating_field->field_type );
    }
    return $display;
}


/**
 * Return the html for a table row displaying the item with entered scores
 *
 * @param  $args   array of named args
 *   atts          array               attributes for the shortcode
 *   item_id       integer             id of item to be rated
 *   rating_fields array of Ewz_Rating_Field    The rating_fields to be displayed
 *   rating_form   Ewz_Rating_Form     The rating_form in which the field is to be displayed. 
 *   ratings       array of Ewz_Item_Ratings   All ratings for the item
 *   rowcount      integer             Total number of rows
 *   rownum        integer             The row number of the item
 *   user_id       integer             id of judge
 *       
 * @return  html for the table row
 **/
function ewz_rated_item_row( $args ){       // 
    // assert( is_array($args) );
    $atts = $args['atts'];
    $item_id = $args['item_id'];
    $rating_fields = $args['rating_fields'];
    $rating_form = $args['rating_form'];
    $ratings = $args['ratings'];
    $rowcount = $args['rowcount'] ;
    $rownum = $args['rownum'] ;
    $user_id = $args['user_id']; 
    $thumb_w = $args['thumb_w']; 

    // assert( Ewz_Base::is_nn_int( $user_id ) );
    // assert( is_a( $rating_form, 'Ewz_Rating_Form' ) );
    // assert( is_array( $rating_fields ) );
    // assert( is_array( $ratings) );
    // assert( is_int( $rownum ) );
    // assert( is_int( $rowcount ) );
    // assert( is_array( $atts ) );
    // assert( Ewz_Base::is_nn_int( $atts['rf_num'] ) );

    $judge_index = 999;
    foreach( $ratings as $n => $rtg ){
        // if the rating has a judge_id that matches the current user, set the judge_index
        if( $rtg->judge_id == $user_id ){
            $judge_index = $n;
        }
    }

    // set row class to 'ewz_new' if this is not a read view and the judge is not finished
    $_attr = ( $atts['view'] == 'read' || isset($ratings[$judge_index]) && $ratings[$judge_index]->complete ) ? '' : " class='ewz_new'";
    $_rid = esc_attr( $atts['rf_num'] . "_$item_id" );
           
    $output ="<tr id='row$_rid' $_attr>";
    foreach ( $rating_fields as $n => $rating_field ) {
        $next_displayed = ewz_next_displayed_field( $atts['view'], $rating_fields, $n );
        if( $next_displayed ){
            $append_next = $next_displayed->append ;
        } else {
            $append_next = false;
        }
        $output .= ewz_display_rated_item_field( array( 'user_id' => $user_id, 
                                                    'rating_form' => $rating_form, 
                                                   'rating_field' => $rating_field, 
                                                        'item_id' => $item_id, 
                                                        'ratings' => $ratings, 
                                                         'rownum' => $rownum, 
                                                    'append_next' => $append_next,
                                                        'thumb_w' => $thumb_w, 
                                                    'judge_index' => $judge_index,
                                                           'atts' => $atts ) 
                                               );
    }

    $output .= '<td>';
    // if this is not a read view show the input fields
    if( $atts['view'] != "read" &&
        count( array_filter( $rating_fields, function($v) { return !in_array( $v->field_type, array( 'fix','xtr', 'lab') );}  ) ) > 0 ){
        // show the input fields
        if( isset( $ratings[$judge_index] ) ){
            $item_rating = $ratings[$judge_index];
            $disable_clear = $item_rating->item_rating_id ? '' : ' disabled="disabled"';
            $_rfnum = esc_attr( $atts['rf_num'] );
            $_item_id =  esc_attr( $item_rating->item_id );
            $nonce_field = wp_nonce_field( 'ewzrating', 'ewzratingnonce', true, false );
            $output .= str_replace( 'id="ewzratingnonce"', 'id="ewzratingnonce'  . $_rfnum . '_' . $_item_id . '"', $nonce_field );
            $output .= '<input type="hidden" name="item_rating_id" value="' .  esc_attr( $item_rating->item_rating_id ) . '">';
            $output .= '<input type="hidden" name="rating_form_id" value="' .  esc_attr( $item_rating->rating_form_id ) . '">';
            $output .= '<input type="hidden" name="item_id" value="' .  $_item_id . '">';
            $output .= '<button type="button" id="ewz_savebtn' . $_rfnum . '_' . esc_attr( $item_rating->item_id ) . '" class="ewz_savebtn_' . $_item_id . 
                '"' . " onClick='ewz_save_changed_rows( " .  $_rfnum . ", this )' disabled='disabled'>Save All</button>";

            $output .= '<br><br><button type="button" id="ewz_clearbtn' . $_rfnum . '_' . esc_attr( $item_rating->item_id ) . '" class="ewz_clearbtn_' . $_item_id . 
                '"' . " onClick='ewz_clear_row( " .  $_rfnum . ", this )'  $disable_clear >Clear Item</button>";
        }
    }
    // last column contains hidden div for the html for the image page, so is always there
    $settings = $rating_form->rating_scheme->settings;
    $item = new Ewz_Item($item_id);
    $output .= ewz_item_rating_page( $rating_fields, $item, $rownum, $settings, $rowcount, $atts ); 
    $output .= '</td>';

    $output .= '</tr>';
    return $output;
}   // end rated_item_row

function ewz_next_displayed_field( $view_att, $rating_fields, $n ){
    // assert( is_string( $view_att ) );
    // assert( is_array( $rating_fields ) );
    // assert( is_int($n));

    $i = 1;
    if( $view_att == 'secondary' ){
        if( isset($rating_fields[$n + $i]) ){
            return $rating_fields[$n + $i];
        } else {
            return false;
        }
    } else {
        while( isset($rating_fields[$n + $i]) ){
            if( !($rating_fields[$n + $i]->is_second) ){
                return $rating_fields[$n + $i];
            }
            ++$i;
        }
        return false;
    }
}        
        

/**
 * Return the html for the list display
 * Called from ewz-rating-shortcode.php
 *
 * @param   integer          $user_id       --  judge user id
 * @param   array            $item_ratings  --  contents of an item_rating table row
 * @param   Ewz_Rating_Form  $rating_form
 *
 * @return  string  $output is the html
 **/
function ewz_rating_list( $user_id, $item_ratings, $rating_form, $atts, $multi_shortcodes ) {
    // assert( Ewz_Base::is_nn_int( $user_id ) );
    // assert( is_array( $item_ratings ) );
    // assert( is_a( $rating_form, 'Ewz_Rating_Form' ) );
    // assert( Ewz_Base::is_nn_int( $atts['rf_num'] ) );
    // assert( is_bool( $multi_shortcodes ) );
    // generate the output
    $num_done = (int)Ewz_Item_Rating::get_judge_count( $user_id, $rating_form->rating_form_id );
    $output = '';
    // show the counts if "rate" view, settings ask for it and rating all items
    // ( counts only show how many items in the rating form have ratings for the judge)
    if( $atts['view'] == 'rate'  && !$atts['item_ids'] && $rating_form->rating_scheme->settings['summary'] ){
        $output .= "<div class='ewz_jstatus' id='ewz_jstatus'>";
        $output .=  "$num_done Items saved.<br>" . (int)$rating_form->rating_status[ 'count' ] . " Items total." ;
        $output .= "</div>";
    }

    $_rfnum =  esc_attr( $atts['rf_num'] );
    $output .= ewz_generate_head_area($_rfnum,$multi_shortcodes,$atts,$rating_form,$num_done);

    $rating_fields = array_values( $rating_form->rating_scheme->fields );
    
    $output .= ewz_generate_judge_options($user_id, $_rfnum, $rating_fields);

 

    if ( count( $item_ratings ) > 0 ) {
        // doing 'submit' via ajax, no form needed 
        $_classlist = "ewz_rtable ewz_rating_table  sortable";
        $output .= '<div class="ratingdiv"><table id="ewz_rtable_' . $_rfnum  . '" class="' . $_classlist .'"><thead>';
        // header row
        $output .= "\n   <tr>";
        $lastfield = count($rating_fields) - 1;
        $thumb_w = ewz_thumbfile_dimensions()['w'];
        foreach ( $rating_fields as $n => $rating_field ) {
            $next_displayed = ewz_next_displayed_field( $atts['view'], $rating_fields, $n );
            if( $next_displayed ){
                $append_next = $next_displayed->append ;
            } else {
                $append_next = false;
            }

            if( $rating_field->is_second && $atts['view'] != 'secondary' ){
                continue;
            }
            if( !$rating_field->append ){ 
                $output .= "<th ";
                      

                if( $rating_field->field_type == 'fix' ){
                    $field = $rating_form->rating_scheme->item_layout->fields[$rating_field->fdata['field_id']]; 
                    // no sorting for image fields or in case of multiple shortcodes on a page
                    if( $field->field_type == 'img' ){
                        $output .= ' class="unsortable" width="' . $thumb_w . 'px"';
                    } else {
                        $output .= ' class="col' . esc_attr( $rating_field->rating_field_id ) . '" ';
                    }
                } else {
                    if( ($rating_field->is_second && $atts['view'] != 'secondary')  ){
                        $output .= ' class="unsortable"';
                    } else {
                        $output .= ' class="col' . esc_attr( $rating_field->rating_field_id ) . '" ';
                     }
                }
                $output .= ">";
            }

            $output .= ewz_header( $n, $rating_field, $rating_fields, $rating_form, $atts );
 
            if( ( $n < $lastfield ) && $next_displayed && $next_displayed->append ) {
                $output .= " / ";
            } else {
                $output .= "</th>";  
            }
        }

        // always needed for "save" column which contains the code for the large window
        $output .=    '<th class="unsortable"></th></tr>';
         
        $output .= '</thead><tbody>';


        // table rows
        $nratings = count( $item_ratings );
        $m = 0;
        foreach ( $item_ratings as  $item_id => $ratings ) {
            // one for each rating, and one with a blank rating if item not rated
            // assert( $m < count($item_ratings));
            $output .= "\n" . ewz_rated_item_row( array( 'user_id' => $user_id, 
            'rating_form' => $rating_form, 
            'rating_fields' => $rating_fields, 
            'item_id' => $item_id, 
            'ratings' => $ratings, 
            'rownum' => $m, 
            'rowcount' => $nratings,
            'thumb_w' => $thumb_w,
            'atts' => $atts ) );
            ++$m;
        }
        $output .= "</tbody></table></div>";
        if( !$atts['rf_num'] && !$atts['item_ids']){
            $output .= "<div class='ewz_bottomtext' id='bottom_p'>";
            $output .=     "<span class='ewzleft'><a id='top_link' href='#columns_top'>Back to the top of the page</a></span> ";
            $output .=     "<span class='ewzrt'><a id='reload_link' href='javascript:window.location.reload(true);'>Cancel Current Changes and Reload Page</a></span>";
            $output .= "</div>";
        }
    }       
    return  $output;
}

function ewz_generate_judge_options($user_id, $_rfnum, $rating_fields)
{
    $str = '';
    $check_radio = get_user_meta( $user_id, 'ewz_useradio', true) ? 'checked' : '';
    $nextafter = get_user_meta( $user_id, 'ewz_nextafter', true);
    if(!$nextafter ){
       $nextafter = 0;
    }
    $esc_rfnum = esc_html($_rfnum);
    $str .= '<div class="ewz_select_to_radio"> ';
    $str .= '    <div class="ewz_small">Some settings to reduce the number of clicks required:</div> ';
    $str .= '    Show drop-down selections as buttons: &nbsp; ';
    $str .= "    <input type='checkbox' id='useradio$esc_rfnum' $check_radio onChange='ewz_selectorradio($esc_rfnum)'>";
    $str .= "    <br>Go to next item after setting &nbsp; <select id='nextafter$esc_rfnum' onChange='ewz_setnextafter(this)'>";
    $str .= "    <option value=0>&nbsp</option>";
    foreach ( $rating_fields as $rfield ){
        if( in_array( $rfield->field_type, array('opt','chk') )){
            $selected = (($nextafter == $rfield->rating_field_id) ? 'selected' : '');
            $str .= "<option $selected value=$rfield->rating_field_id>".esc_html($rfield->field_header)."</option>";
        }
    }
    $str .= "</select><br>";
    $str .= "<div class='ewz_prefbtn'><button  id='ewzjudgeprefs$esc_rfnum' onClick='ewz_save_judge_prefs($user_id,$esc_rfnum)'>Save These Settings</button></div>";
    $str .= "</div>";

    return $str;
}
function ewz_generate_head_area($_rfnum,$multi_shortcodes,$atts,$rating_form,$num_done){
        
    $esc_rfnum = esc_html($_rfnum);
     $str = '<table class="ewz_chead" id="ewz_chead' .  $esc_rfnum  . '" ><tr>';

    // $atts['finish'] is blank, show or hide.  show or hide overrides scheme settings.
    if( isset( $atts['finish'] ) && $atts['finish'] ){
        $show_finished =  ( $atts['finish'] == 'show' );
    } else {
        $show_finished = ( $atts['view'] == 'rate' && $rating_form->rating_scheme->settings['finished'] ) ||
            ( $atts['view'] == 'read' && $rating_form->rating_scheme->settings['finished_r'] ) ||
            ( $atts['view'] == 'secondary' && $rating_form->rating_scheme->settings['finished_s'] );
    }
    
    // Show "finished" button if $atts or settings ask for it ( even in a read view, if admin wants to know if it's been seen )
    // $atts overrides scheme settings
    if( $show_finished ){
        $str .= "<td rowspan='2' class='ewz_jdone' >";
        $str .=    "When you have finished <u>all</u> the items in '$rating_form->rating_form_title', please click the button ";
        $str .=    "below to notify the administrators of this.  The software will first make sure you have actually ";
        $str .=    "saved everything correctly, then remove your access to the page. ";
        $str .=    "<br><button type='button' id='donebutton" . $esc_rfnum . "' onClick='ewz_judge_finished( " .  $esc_rfnum  . ")'>Finished '$rating_form->rating_form_title'</button>";
        $str .= "</td>";
    }
    
    // show help button if settings ask for it
    if( $rating_form->rating_scheme->settings['jhelp']){   
        $max_shrink = $rating_form->rating_scheme->settings['max_shrink'];
        $str .=  '<td><button type="button" id="help_btn' . $esc_rfnum . '" onClick="ewz_get_judge_help(' .$esc_rfnum . ' )">Using This Rating Form</button>';
        $str .=      '<div id="ewz_jhelp'. $esc_rfnum . '"  class="wp-dialog ewz-dialog">' . ewz_jhelp_text($atts, $multi_shortcodes, $max_shrink) . '</div>';
        $str .=  '</td>';
    }

    // show "next unrated" button only if showing all items and some but not all have ratings for the judge
    if( $atts['view'] != 'read' && !$atts['item_ids'] && ( $num_done > 0 ) && ( $num_done < $rating_form->rating_status[ 'count' ] ) ){
        $str .= '</tr><tr><td><button type="button" id="next_new_btn' . $esc_rfnum . '" onClick="ewz_to_next_item( ' . $esc_rfnum . ' )">Jump To First Unrated Item</button></td>';
    }

    $str .=  '</tr></table>';  // chead
    return $str;

}
function ewz_jhelp_text($atts, $multi_shortcodes, $max_shrink){
    // assert( is_array($atts) );
    // assert( is_bool($multi_shortcodes) );
    // assert( is_int( $max_shrink ) && 0 <= $max_shrink && $max_shrink <= 100 );

    $help_str1 = <<<EOT1
        &nbsp; 
        <p><u>If this is your first time using the interface, please read these instructions carefully before proceeding.</u>
              ( This window is resizeable ).
        </p>
        <p>For repeat users, here are the really important points to remember: </p>
        <ul class="ewz_lpad">
           <li>If you <u>leave it for any length of time</u>, or if you see any <u>error messages you do not understand</u>, then be sure to <u>reload the page before continuing</u>.  This will refresh your permission status, and show you exactly what has been saved on the server.</li>
           <li>To refresh/reload the page:
                 <ul class="ewz_lpad">
                    <li>Windows: ctrl + F5</li>
                    <li>Mac/Apple: Apple + R or command + R</li>
                    <li>Linux: F5</li>
                 </ul>
           </li>
           <li>Make sure your image window is on a calibrated monitor and is <u>large enough to show the test image completely</u>.</li>
           <li>Make sure you do not have a default zoom level set in your browser.</li>
           <li>"Save" buttons are disabled until you make a change.</li>
           <li>Save frequently. You can change anything later.</li>
           <li>If scoring, use the sort facility to review your scores before finishing.</li>
           <li>As of version 1.1.10, left and right arrow keys should navigate between items, except when you are inside a text input.</li>
       </ul>
EOT1;

    $resize_string = ' or shrunk. ';
    if( $max_shrink < 100 ){
        $resize_string =  ", since this may decrease sharpness. </p><p>If necessary to fit on your screen, image dimensions may be reduced to a limited extent. ".
                          "But to avoid artefacts, they may not be reduced below {$max_shrink}% of their original value. If they are reduced, a button will " .
                          "appear to allow toggling between the reduced-size view and the full-size view with scrollbars.";
    }
    $help_str1A = <<<EOT1A
        <h2>Display Settings</h2>
    As of version 1.1.16, two new settings have been added, with the aim of reducing the number of mouse clicks required for a large judging.<br>
    1. Drop-down items may be changed to look like a series of buttons.<br>
    2. You may pick a special checkbox or dropdown column. After you change a value in this column, the "current row" is automatically  advanced to the following one.<br>
    With these two settings on a (dropdown) score column, only one click per item is required to score them all.
    <p>These settings may be saved (for each judge separately). If you do not click "Save These Settings", they will revert to the default when the page is reloaded.<br>
    The "buttons" option does make the page a lot longer, so if you use and save it, you may wish to uncheck it temporarily for a final overview.  The view will revert to your saved one on the next reload.

EOT1A;

    $help_str2 = <<<EOT2
        <h2>Setting Up the Windows</h2>
    <p>The first time you click on a thumbnail, a new window will appear showing a test image.  The purpose of this test image
       is to allow you to resize and re-position the window to ensure all images will be seen properly. 
    </p>
    <p>Once such a window exists, all subsequent images will display in this same window. We will refer to it as the
    "image window" and to the original window as the "list window". If you can, <u>resize the list window</u> 
    as needed and <u>arrange the two windows</u> so both are visible and do not overlap. 
    Clicking on the 'OK' button below the test image will then display a larger version of the thumbnail you clicked.  
    </p>
    <p>If you are using multiple screens, the image window should be on one that is color-managed.  
    </p>
   <h2>Image Dimensions</h2>
    <p>To ensure that all users see the same view, without blurring or distortion, images may <b>not</b> be enlarged$resize_string
        </p> 
    <p>You may re-view the test image at any time by clicking the "Display A Test Image" link in the bottom of the image window. If the test image is 
       cut off in any way, ( i.e. if you cannot see the entire border ), enlarge the window until it fits and the "Back" link below the image is visible.
       Make quite sure your browser does not have a default zoom setting -- if it does, the window may be showing an enlarged view.
    </p>
EOT2;
    if( !$multi_shortcodes ){
        $help_str2 .= <<<EOT2b
            <h2>Sorting</h2>
        <p>Clicking on any column header containing an up or down arrow should sort the items by that column.
        </p>
        <p>Refreshing/Reloading the page should return the items to their original order.
        To refresh/reload the page:
            <ul class="ewz_lpad">
               <li>Windows: ctrl + F5</li>
               <li>Mac/Apple: Apple + R or command + R</li>
               <li>Linux: F5</li>
            </ul>          
        </p>
EOT2b;
    }
    if( $atts['others'] ){
        $help_str2 .= '<h2>Other Judges</h2>Any existing inputs from other judges are displayed for your information. Each is prefixed by a number identifying the judge.';
    }
    $help_str3 = <<<EOT3
        <h2>Saving and Errors</h2>
    <p>Each row ( item ) has a "Save" button at the end. This button is <u>disabled at first</u>.
    As soon as you make any change to an item, the Save button is enabled and gets a red border. 
        </p>
    <p>Clicking <b>any</b> enabled "Save" button saves all the items with  enabled "Save" buttons -- i.e all the items
    changed since the last save.
        Once saving is complete, the red border around the "Save" buttons disappears, and the buttons are greyed out, indicating
        they are no longer enabled.  They will be re-enabled if you make any further change.
                                                                        The black border around the item row, which indicates you have no rating saved for the item, also disappears.
                                                                                                                                                                   </p>
    <p>The total number of ratings saved is displayed in the bottom left-hand corner of the list window. It should
    update after each save.
        </p>
    <p>It is suggested that you should save quite frequently, just in case of mistakes or network failures. You can always
    go back and change any item later.
        </p> 
    <h2>Errors and Messages</h2>
    <p>If there is a required field that you failed to fill out, no items will be saved. Instead, an error message will pop 
    up, and the row with the missing/incorrect item will be centered in the screen and highlighted. Once
        all errors have been corrected, all items with "Save" enabled will be saved.
        </p>
    <p>To speed things up, individual saves do not reload the entire page.
    But if you leave the window open for an extended period, <u>you should occasionally completely reload the page after saving</u>. 
    Otherwise, security features in Wordpress may cause your authorization to expire, and you will lose any unsaved work.
    If you leave the window open for long enough, you may see a reminder to do this. <br>A link to reload the page is provided at 
                                                                                                                               the bottom.
                                                                                                                               </p>
    <p> &nbsp;
    </p>
EOT3;

    if( $atts['view'] == 'read' ){
        return $help_str2;
    } elseif( $atts['view'] == 'rate' ){
        return $help_str1 . $help_str1A . $help_str2 . $help_str3;
    } else {
        return $help_str1 . $help_str2 . $help_str3;
    }
}


 
/**
 * Return the table header row
 *
 * @param $n             int                          index of the rating_field in the rating_fields array
 * @param $rating_field  Ewz_Rating_Field             the field to be displayed in the column
 * @param $rating_fields array of Ewz_Rating_Fields   all the fields ( needed in case of append )
 * @param $rating_form   Ewz_Rating_Form              the rating form being displayed
 * @param $atts          array of strings             attributes set in the shortcode invocation
 **/
function ewz_header( $n, $rating_field, $rating_fields, $rating_form, $atts ){
    // assert( is_int( $n ) );
    // assert( is_a( $rating_field, 'Ewz_Rating_Field' ) );
    // assert( is_array( $rating_fields ) );
    // assert( is_a( $rating_form, 'Ewz_Rating_Form' ) );
    // assert( is_array( $atts ) );
    $output = '';
    // do not display an "is_second" field unless the view is "secondary" 
    // TODO:  should it also be shown in a read-only view?
    if ( $rating_field->is_second &&  ( "secondary" != $atts['view'] ) ) {
        return '';
    }
    if( $rating_field->required ){
        $output .= '*';
    }
    $output .=  esc_html( $rating_field->field_header );
    if( in_array($rating_field->field_type, array('opt','rad','chk') ) ){
        $chkid = "jump" . $rating_field->rating_field_id;
        $output .= "<br><input type='checkbox' id='$chkid'  onChange='set_jump_column($rating_field->rating_field_id)'>";
    }
    return $output;
}
